# from hashmark # to end of line = comment

# R Comments identified with '#' prefix

# Lab 1: Introduction to the Nuts and Bolts of R
# Add-on packages can be installed individually via Tools:Install Packages menu or install.packages() command:
install.packages(c("car","Hmisc","pROC","effects","gmodels","psych", "readxl", "EnvStats"))

# authors of research articles in scientific journals now appear to overwhelmingly employ R for executing and displaying published statistical results. Joseph M. Hilbe, Journal of Statistical Software, Sept 2010

# Useful links
# Comprehensive R Archive Network http://cran.r-project.org
# On-line R support: RSeek (www.rseek.org), UPennsylvania user forums, RPubs
# Quick-R site: http://www.statmethods.net
# The R Book 2013, by Michael Crawley http://primo-pmtna01.hosted.exlibrisgroup.com/UMB:UManitoba:TN_wilbookl10.1002/9781118448908
# Introductory Statistics with R by Peter Dalgaard
# Introduction to R Commander: https://cran.r-project.org/doc/contrib/Karp-Rcommander-intro.pdf

# RStudio
## review windows: Console, Editor, Environment, History, Plots, Packages (e.g. psych), Help


 # arithmetic operations in calculator mode
 3+4    # addition
 2*6    # multiplication
 3/4    # division
 3^4    # exponentiation
 log(81)     # base e=2.718 logarithm
 exp(4.394)  # inverse of base e logarithm ('anti-log')
 log10(81)   # base 10 logarithm
 10^1.90849  # inverse of base 10 logarithm
 round(pi,2) # round pi = 3.1416 to 2 decimal places
 sin(pi/2)   # sine of pi, where pi = 3.1416 radians
 cos(pi/4)   # cosine(pi/4)


 # order of operations
 3 + 2 * 5 # multiply first
 (3 + 2) * 5 # add first


 # assignment of column vectors x
 x=c(1,2,3,4,5,6,7,8,9,10,11,12)
 x<-c(1,2,3,4,5,6,7,8,9,10,11,12)
 c(1,2,3,4,5,6,7,8,9,10,11,12)->x

 # note that nothing happens when you assign a variable name, unless you ask to see its value
 x  # display x
 print(x)  # equivalent to x


 # most functions operate on vectors
 mean(x) # mean of x

 # missing data indicator: NA vs . vs 99999
 x_missing=c(1,2,3,NA,5,6,7,8,9,10);x_missing
 mean(x_missing)
 mean(x_missing, na.rm=TRUE)
 sd(x) # standard deviation of x
 sd(x_missing, na.rm=TRUE)
 length(x) # number of elements in vector x


 # most operations on vectors act element-wise
 x/2 # divide each element of x by 2
 log(x) # take log (base e) of each element in x
 y=x^2;y  # square each element in x, semicolon starts a new line

 # functions for character vectors
 # http://www.statmethods.net/management/functions.html
 pcode=c("B2N 3V9", "R3M 0C3", "H4A 2L1", "R0P 1C9");pcode
 nchar(pcode)  # number of characters in each element of vector
 substr("R3M 0C3",1,3) # first 3 characters of string
 substr(pcode, 1, 3) # first 3 characters from each element of vector
 # paste two strings together to eliminate space (PCCF)
 paste(substr(pcode, 1,3), substr(pcode,5,7), sep="")

 # Help functions
 ?mean #  search active workspace (base R, loaded packages) for help on 'mean'
 ??mean #  search all installed packages
 apropos("mean") # fuzzy search
 # in RStudio, same by starting to type name followed by tab e.g. me...

 help(package="readxl") # help on a specific package
 library(readxl) # load package into active workspace = require(readxl)

 help(package="car") # help on the car package (Fox)
 citation(package="car")
 library(car) # load car library

 x=c(1,2,3,4,5,6,7,8,9,10);x # define and display vector x ';'
 # alternatively, create x via seq() function
 x=seq(from=1, to=10, by=1);x # or seq(1,10,1) -  order matters
 y=x^2;y # y = x squared
 z=x^3;z # z = x cubed

 # create data frame using original column names
 d=data.frame(x, y, z)
 head(d) # display first 6 rows

 # re-name the columns as the data frame created
 d1=data.frame(x1=x, x2=y, x3=z)
 tail(d1) # display last 6 rows

 ## Subsetting data
 y[4] # extracting elements by index, where elements range from 1 - length(y)

 # logical comparison operators applied to each element -> TRUE/FALSE
 y == 9 # is y_i equal to x (note double ==)
 y < 9 # each element of y is compared to 9 -> TRUE/ FALSE

 # extract elements of y by TRUE/ FALSE
 y[y < 9] # using TRUE-FALSE index

 # extract subsets of data using subset()
 y2=subset(y, y<9);y2

 rm(x,y,z) # remove x,y,z from top-level workspace
 x         # x no longer exists except inside the data frame
 head(d)   # display first 6 rows of d[,c("x","y","z")]
 d$x       # display variable x from the dataframe d, using $ slot extractor

# To display data table, click d in Env window or
 View(d) # show data

 ## subset by row and column indices [rows, columns]
 d[5,1] # element at row 5, column 1
 d[5,]  # row 5 by row index
 d[,1]  # column 1 d by column index
 d[, c(1,3)]  # columns 1 and 3 by indices
 d[, -c(1,3)] # delete columns 1 and 3

 d[,"x"] # column 1 by name
 d[, c("x","z")] # columns 1 and 3 by name
 d$x  #column 1 via slot operator $


 # subsetting by logical comparisons (==, !=, <, >)
 d2=d[d$x==5, ];d2 # all columns where x == 5
 d2=d[d$x!=5, ];d2 # all columns where x != 5
 d3=d[d$x<5, ];d3 # all columns where x < 5

 d4=subset(d, x<5);d4 # d where x < 5
 d5=subset(d, x>2 & x<5);d5 # d where x > 2 AND x < 5
 d6=subset(d, x==5 | x==10);d6 # d where x = 5 OR x = 10
 d7=subset(d, x %in% c(5,10));d7 # d where x = in c(5, 10)

 # formula notation y ~ x in plot() or models
 plot(y~x, data=d)
 plot(y~x, data=d, type="p") # point plot
 plot(y~x, data=d, type="l") # line plot
 plot(y~x, data=d, type="b") # both points and lines
 plot(y~x, data=d, type="s") # stair plot
 plot(y~x, data=d, type="h") # high density vertical lines

 # building plots by layers
 plot(y~x, data=d, type="h") # vertical line plot
 points(y~x, data=d) # add point layer to existing plot
 lines(y~x, data=d, lty="dashed") # add line layer to plot (lty = line type)

 ?plot # many options e.g. xlab, ylab, main, xlim, ylim
 plot(z~x, type="b", data=d, xlab="X", ylab="Z = X^3", main = "Cubic")
 plot(z~x, type="b", data=d, xlab="X", ylab="Z = X^3", main = "Zoomed", xlim=c(2,6), ylim=c(0,250))

# plot colors (col) and characters (pch)
 library(Hmisc) # Frank Harrell's miscellaneous package
 show.col()
 show.pch()

# dressing up a plot with options, layers, legends
 plot(z~x, data=d, type="b", pch=19,  col="red", lty="dashed", ylab="", xlab="X", main="Curvilinear Demo")  # lty=2 or lty="dashed" for dashed line

 # add another layer
 points(y~x, data=d, pch=21, col="blue", type="b", lty="solid")
 #add a legend
 legend("topleft", pch=c(19,21), lty=c("dashed","solid"), col=c("red","blue"), legend=c("Cubic","Quadratic"), bty='n')

 # formula notation y ~ x in regression of y on x
 model1=lm(y~x, data=d) # univariate linear regression

 # model1 is an OOP object with methods (functions) like summary() or plot()
 summary(model1) # most R objects respond to summary
 plot(model1) # rudimentary residual diagnostics

 # model1 is an OOP object with properties (attributes)
 names(model1) # lists the built-in properties associated with any object
 model1$fitted.values # predicted values property ($ = slot extractor)
 model1$residuals # residuals (observed - predicted ) property


 ## IMPORTING EXTERNAL DATA: data dictionary (missing NA, not '.', 99999)
 # bwt = birth weight in grams (continuous).
 # low = low birth weight indicator: low vs norm (factor, reference  = low)
 # age = mother's age in years (continuous).
 # lwt = mother's weight in pounds at last menstrual period (continuous).
 # race2 = mother's race (white vs nonwhite) (factor, reference = nonwhite)
 # race3 = mother's race (white vs black vs other) (factor, reference = black)
 # smoke = smoking status during pregnancy  yes vs no (factor, reference=no)
 # ptd = history of preterm delivery yes vs no (factor, reference=no)
 # ht = maternal hypertension yes vs no (factor, reference=no)
 # ui = presence of uterine irritability yes vs no (factor, reference=no)
 # ftv = number of physician visits during the first trimester (continuous, 0-6)

## First set working directory (Sessions:Set Working Directory)
 getwd()
 d=read.csv("bwt.csv") # read .csv spreadsheet
 head(d) # inspect first 6 rows
 tail(d) # inspect last 6 rows
 View(d) # == click on dataset name in Environment panel

 # Useful dataframe functions
 dim(d) # 189 rows, 11 columns - like length() for multi-dimensional data)
 nrow(d) # number of rows
 ncol(d) # number of variables columns
 data.class(d) # what type of object is d (data.frame)
 sapply(d, data.class) # sapply applies data.class function to each column of d

  # reading .xls/.xlsx sheets directly
 library("readxl") # load package readxl
 excel_sheets("birthwt.xls") # identify worksheets inside workbook
 head(read_excel("birthwt.xls", sheet=1)) # read and display sheet 1

 summary(d) # summarize numeric, categorical (factor) variables

  # specify data types at import
 d=read.csv(file="bwt.csv", header=TRUE, colClasses=c("numeric","factor","numeric","numeric","factor","factor","numeric","factor","factor","factor","factor"))
 sapply(d, data.class) # sapply applies data.class function to each column

 # type-casting
 d$ftv_cat=as.factor(d$ftv) # cast numeric -> factor
 d$ftv_cat=factor(d$ftv_cat, ordered=TRUE) # cast numeric -> ordinal factor
 summary(d)  # new column added
# type casting functions: as.numeric, as.factor, as.character, as.logical, as.dataframe, as.vector,as.Date.
# analogous type-checking functions that return True-False e.g. is.numeric(), is.factor(), etc.

 unclass(d$race3) # factor -> numbers, useful for plot colors/symbols

# categorize continuous -> factor by intervals e.g. (0, 1500]  >0, ≤1500
 d$bwt_cat=cut(d$bwt, breaks=c(0, 1500, 2500, 4000, Inf), labels=c("VLBW","LBW", "Norm","LGA"));
 data.class(d$bwt_cat) # confirm type of new column

 # R supports simultaneous datasets -> d$bwt to specify dataset
 attach(d)  # convenient if working with only one dataset
 summary(bwt)  # no longer need to specify the dataframe
 detach(d)  # detach to avoid name clashes

 with(d, summary(bwt)) # one dataset for all variables inside with(d, )

 levels(d$low) # factor levels ordered alphabetically (cf SAS)
 levels(d$race3)

 summary(d[ , c(2,5,6,8,9,10,11)]) # extract factors by indices [..]

 d$low=factor(d$low, levels=c("norm","lbw")) # re-order so norm is reference
 levels(d$low)

 levels(d$race3)
 d$race3=relevel(d$race3, ref="white") # re-order so whites are reference
 levels(d$race3)

 d$ftv_cat=factor(d$ftv_cat, ordered=TRUE)
 d$ftv_cat # levels are now ordered, for regression


 ###########################
#R is a __fully functional__ programming language, with support for if-then-else comparisons, for-loops, case statements, debugging tools, and other standard programming constructs.For examples, check out our pages for plotting growth charts and calculating pediatric Z-scores at http://apps.cpeg-gcep.net/

# extracts numeric columns from dataframe
numCols<-function(data){
if (is.data.frame(data)){
  return(data[,sapply(data,  data.class)=="numeric"])}}

# extracts factor columns from dataframe
facCols<-function(data){
if (is.data.frame(data)){
  tf<-sapply(data,  data.class) =="factor" |sapply(data, data.class)=="ordered";
  return(data[, tf])}}


# example
d1=numCols(d);head(d1)
d2=facCols(d);head(d2)
###########################

## HOMEWORK EXERCISE
# childiq.csv data dictionary - import, format, summarize

# kid_score: standardized scholastic tests scores: continuous
#
# mom_iq = maternal IQ (Stanford Binet): continuous
#
# mom_hs = maternal highschool completion: factor NoHS vs HS, ref=HS
#
# mom_age: maternal age (years): continuous
#
# mom_work = type of employment 1-4: factor, ref= year1
# year1: full-time in first year of child’s life
# year2: didn’ work during child’s first year
# year3: didn’t work during first 2 years of child’s life
# year4: didn’t work during first 3 years of child’s life
