## Lab 3: Regression Modeling: predictors of child IQ

# Outcome: child IQ score (continuous)
# Primary exposure: maternal IQ (continuous)
# Hypothesis: child IQ is correlated with maternal IQ
# Covariates/confounders: maternal education, maternal work status, maternal age

# Exercise: what type of regression model should be used to study continuous outcome?
# First, set working directory (Sessions: Set Working Directory)
 d=read.csv("childiq.csv", stringsAsFactors=TRUE) 
 #stringsAsFactors=TRUE uses text data to identify levels of a factor (categorical) 
 sapply(d, data.class)
 summary(d)

## Exploratory analysis
 # EMPIRIC DISTRIBUTIONS
 # Maternal IQ
 with(d, hist(mom_iq))
 qqnorm(d$mom_iq) # is it normal, does it matter?
 summary(d$mom_iq)
 sd(d$mom_iq)
 quantile(d$mom_iq, probs=c(0.025, 0.15, 0.5, 0.85, 0.977) )


 # Child scores
 with(d, hist(kid_score)) # is it normal, does it matter?
 summary(d$kid_score)
 # z= -2, -1, 0,  +1, +2
 quantile(d$kid_score, probs=c(0.025, 0.15, 0.5, 0.85, 0.977) )

## bivariate relationships:
 # continuous predictor and continuous outcome
plot(kid_score~mom_iq, data=d)
abline(lm(kid_score~mom_iq, data=d), lty=2)  # add best fit linear
lines(lowess(d$kid_score~d$mom_iq), lty=1) # add lowess smooth
# is this relationship linear or quadratic?

# categorical predictor and continuous outcome
boxplot(kid_score~mom_hs, data=d) # is there a relationship with maternal education?

# one-step bivariate screening
library(car) # first install.packages("car"), or use Tools menu
scatterplotMatrix(d, smooth=F)

## our model will includes both
  # continuous predictors (beta = slope i.e. expected change in y for unti change in x)
  # categorical predictors or factors (beta = shift in intercept as dummy x=0 moves to x = 1)
## For categorical vars - what is the reference level?
# depends on type of contrast

 # - Treatment Coding: Compares each level of a variable to the reference level
 # - Forward Difference Coding: Compares adjacent levels of a variable (each level minus the next level)
 # - Backward Difference Coding: Compares adjacent levels of a variable (each level minus the prior level)
 # - Helmert Coding: Compare levels of a variable with the mean of the subsequent levels
 # - Reverse Helmert Coding: Compares levels of a variable with the mean of the previous levels
 # - Deviation Coding: Compares deviations from the grand mean
 # - Orthogonal Polynomial Coding: Orthogonal polynomial contrasts (ordinal categorical vars)
 # - User-Defined Coding:	User supplies a custom contrast matrices

# by default, R uses 'treatment contrasts' for unordered factor variables
 options("contrasts")

 # display contrast matrix (k levels-> k-1 dummies)
 contrasts(d$mom_hs) # 2 levels 1 dummy, HS = reference category
 contrasts(d$mom_work) # 4 levels 3 dummies, year1 = reference category

 # bivariate screening via single predictor models
 lm1<-lm(kid_score~mom_iq, data=d);summary(lm1) # obligate predictor - why?
 # interpret beta0 (intercept), beta1, and R2
 lm2<-lm(kid_score~mom_hs, data=d);summary(lm2)  # include in final model?
 lm3<-lm(kid_score~mom_age, data=d);summary(lm3) # include in final model?
 lm4<-lm(kid_score~mom_work, data=d);summary(lm4) # include in final model?

 # EXERCISE explain lm4: Is this likelty to be spurious or real?

 # main effects model: kid_score vs mom_iq, mom_age, mom_hs
 lm_main<-lm(kid_score~mom_iq + mom_hs + mom_age, data=d);summary(lm_main)
 # what happened to the p-value for mom_age?
 # interpret R2

 # add non-linear term, note I()
 lm_quad<-lm(kid_score~mom_iq + I(mom_iq^2) + mom_hs + mom_age, data=d);summary(lm_quad)
 # interpret R2

 # likelihood ratio test to compare two models
 anova(lm_main, lm_quad, test="F")
 # which model is better
 # R2 0.22 -> 0.23 is it worth adding quadratic term?

 # main effects = slope common to all subgroups
 # define 'interaction term' ?
 # interaction beween maternal education and IQ: slopes differ: slope of child vs maternal IQ relationship is different between moms with HS and moms without hs. What's the easiest way to test this?

 # Stratified analysis to test for interaction between mom_iq and mom_hs
 lm5_NoHS<-lm(kid_score~mom_iq, data=subset(d, mom_hs=="NoHS"));summary(lm5_NoHS)
 lm5_HS<-lm(kid_score~mom_iq, data=subset(d, mom_hs=="HS"));summary(lm5_HS)
# slope 0.48 with HS -> 0.97 without HS. Think about.

 # a graphical test for an interaction?
 # Interaction plot to check for parallel slopes
plot(kid_score ~ mom_iq, data=d, col=unclass(mom_hs), pch=unclass(mom_hs)+19, main="Interaction Plot")
 abline(lm5_HS, lty=1, col=1) # add line for HS graduates
 abline(lm5_NoHS, lty=2, col=2) # add line for HS dropouts
 legend("topleft", pch=c(20,21), col=c(1,2), lty=c(1,2), legend=c("HS", "NoHS"), bty="n")

 # add an interaction effect to main effects model
 lm6<-lm(kid_score~mom_iq + mom_hs + mom_age +  mom_iq:mom_hs, data=d)
 summary(lm6)

 #Exercise
 # Interpret the magnitude and the p-value for the interaction term?
  
#Are you confused by the impact of NoHS education, which implies the intercept is shifted down by 52 points  in children of less educated mothers?  Explain.

# center numeric values to re-position intercept
 d$mom_cage=d$mom_age - mean(d$mom_age) #cage = centered age
 d$mom_ciq=d$mom_iq - mean(d$mom_iq) # ciq = centered IQ

 d$mom_cage=scale(d$mom_age,  center=TRUE, scale=FALSE)
 d$mom_ciq=scale(d$mom_iq,  center=TRUE, scale=FALSE)

 lm6b<-lm(kid_score~mom_ciq + mom_hs + mom_cage +  mom_ciq:mom_hs, data=d)
 summary(lm6b)

# Exercise
# Interpret the new coefficients.

 #Exercise
 #How is the R^2^ (or adjusted R^2^) affected by the addition of the interaction term? How do you decide if it's worth including?  

 confint(lm6) # non-significant if cross zero


 # add quadratic term, using I()
 lm7<-lm(formula = kid_score ~ mom_iq + I(mom_iq^2) +  mom_hs + mom_age +  mom_iq:mom_hs, data = d)
 summary(lm7)


# _All_ modeling functions in R comes with `predict()` functions that allows you to apply the fitted model to either the study sample or a new dataset e.g. to validate the model on external data or plot results for specific (idealized) patient groups:

d$predict1=predict(lm_main) # or predict(model, newdata=..)
error1=d$kid_score - d$predict1 # prediction errors O-E aka residuals
hist(error1) # error distribution
hist(residuals(lm_main)) # errors = residuals

 # likelihood ratio test to compare two models, here with and without mom_hs.
 # EXERCISE Since mom_hs appears twice,these models differ by 2 dof
 # lm6: kid_score ~ mom_iq + mom_hs + mom_age + mom_iq:mom_hs
 lm8=lm(kid_score ~ mom_iq + mom_age, data = d)
 # likelihood ratio test
 anova(lm6, lm8, test="F")
 # Is  model improvement statistically significant? Is R2 0.24 worth the effort?

# Gaussian linear model is special case of GLM (identity link, gaussian errors)
 glm_main<-glm(kid_score ~ mom_iq + mom_hs + mom_age, data=d, family="gaussian")
 summary(glm_main)

 # EXERCISE Regression diagnostics
 # Are residuals normally distributed?
  # Is there evidence of collinearity (risk of type II error i.e. false negative Wald tests)
 vif(lm_main)

 
 ### Logistic Regression Example
 # outcome = low (birthwt weight) indicator, 0 = norm, 1 = lbw = binary categorical
 # primary exposure = race2, white vs non-white
 # hypothesis = LBW more common in non-white familes
 # covariates = "bwt"   "low"   "age"   "lwt"   "race2" "ftv"   "smoke" "ptd"   "ht"    "ui" 
 
d<-read.csv("bwt.csv", stringsAsFactors = TRUE)
summary(d)
# define reference levels for factors
d$low<-factor(d$low, levels=c("norm","lbw"))
d$race2<-factor(d$race2, levels=c("white","nonwhite"))
d$race3<-factor(d$race3, levels=c("white","black","other"))
summary(d)

 # single predictor models (bivariate screening)
 glm1<-glm(low~race2, family="binomial", data=d);summary(glm1)
 exp(glm1$coefficients) # odds ratio through anti-log
 exp(confint(glm1)) # 95% confidence interval on odds ratios ( ?cross 1)

 # Hosmer and Lemeshow build the following main effects model
 glm2<-glm(low~lwt+race2+smoke+ptd+ht, family="binomial", data=d);summary(glm2)
 exp(coefficients(glm2)) # OR's
 # interpret OR and p-values
 confint(glm2) # 95% CI's on beta's
 exp(confint(glm2)) # 95% CI's on ORs
 
 #Prediction Performance p  vs linear predictor: S-shape
 pred2=predict(glm2, type="response"); # original 0-1 scale vs "link" for log(odds)
 plot(pred2~glm2$linear.predictors, xlab="Linear Predictor", ylab="Prob of LBW", pch=19)

 # or use ROC curve to assess discrimination performance (c-statistic)
 library(pROC)
 roc1<-roc(d$low, pred2, plot=TRUE, ci=TRUE, main="low birth weight ROC");roc1
 
 # plot of sensitivity = true positives vs 1- specificity = 1- true negative rate = true negative rate.
 # AUC = measures overall classifier performance from 0.5 (random chance) to 1 (perfect discrimination). THe diagonal line of random chance has AUC 0.5 = no better coin flipping. An AUC of 0.9 = excellent, 0.75 is moderate

 # # add labels
 txt1<-paste0("AUC:",round(roc1$auc,2),"\n95% CI: ", round(roc1$ci[1],2),"-", round(roc1$ci[3],2));txt1
 text(x=0,y=0.05,labels=txt1, cex=0.8)

 # find 'optimal' threshold
 # youden = maximizes sum sensitivity + specificity
 coords(roc1, x="best", best.method="closest.topleft", transpose=TRUE)

##DIAGNOSTICS
 # COLLINEARITY between predictors - risk of type II error
 library(car)
 vif(glm2)

 # # Assess linearity on logit scale (Taylor Series lhs)
 # avPlots(glm2)
 #
 # # EXTREME VALUES INDEX PLOTS:
 # infIndexPlot(glm2, vars=c("Studentized", "hat","Cook"), id.n=3) # 147, 183
 # # Bubble Plot
 # influencePlot(glm2, main="Area proportional to Cook's D", ylim=c(-3,3))
 #
 # # individual influence diagnostics
 # dfbetasPlots(glm2, id.n=3) # id the 3 most influential
 # dfb<- dfbetas(glm2)
 # dfb[c(51, 147),]
 #
 # # how influential?
 # summary(glm(low~lwt+race2+smoke+ptd+ht, family=binomial, data=d[-51,]))
 # summary(glm(low~lwt+race2+smoke+ptd+ht, family=binomial, data=d[-147,]))
 #


 ### Survival Model Example - usually omitted from lab

 # We're going to use duration of breast-feeding (time to stopping) in a survival model, where time to death is replaced by time to stopping breast feeding. The data dictionary (missing data indicator = NA)

 # - duration: continuous, duration of breast feeding, weeks
  ## outcome, right-censored so can't use linear or logistic regression
 # - censor: indicator of completed breast feeding (1=yes, 0=no)
 # - race: factor, maternal race, (1=white, 2=black, 3=other)
    ## primary exposure, categorical with 3 levels
 # Other covariates:
 # - poverty: factor, maternal poverty (no, yes)
 # - smoke: factor, maternal smoking at birth (no, yes)
 # - alcohol: factor, maternal alcohol use at birth (no, yes)
 # - age: continuous, maternal age of mother in years at birth of child
 # - year: continuous, year of birth (78-86)
 # - yschool: continuous, maternal education in years
 # - pc3mth: factor, prenatal care after first trimester (no, yes)

 d=read.csv("bfeed.csv", stringsAsFactors=TRUE)
 # outcome is 'time to cessation of breast-feeding' = continuous right censored
 # censoring precludes linear or logistic regression
 # primary exposure: race, categorical
 summary(d)
 d$race=relevel(d$race, ref="white") # make white the baseline
 dim(d) # 927 subjects (mother-child dyads)
 
 library(survival) # load library
 # create survival object with censored marked (+)
 d$survival=with(d, Surv(time=duration, event=censor));
 head(d,8)

# to compare groups, need a factor (group) variable
 d$fyear=as.factor(d$year) # make factor for grouping
 with(d, table(fyear)) # count by year

 # Kaplan Meier comparison of groups
 km=survfit(survival~fyear, data=d);km

 # test KM curves for significance
 survdiff(survival~fyear, data=d)

 #plot KM curves (confusing with 9 groups)
 plot(km, lty=unclass(d$fyear))
 legend("topright", lty=c(1:9), legend=c(78:86), bty="n")

 # survminer is new KM plotter
 library(survminer)
 ggsurvplot(km)

# EXERCISE What is happening to median 'survival' (breast-feeding duration) over time? Are these differences significantly different from what would be expected under the null hypothesis.

 # EXERCISE: With 9 annual cohorts, these results are hard to interpret. Moreover, we expect temporal trends to evolve gradually over time. In this case, a regression for hazard on year might make more sense.

 # coxph model fitted by Cox Partial Likelihood method
 model1=coxph(survival~year, data=d)
 summary(model1)

# EXERCISE: hazard ratio `exp(coef)` associated with `year` is 1.05 (p $<$ 0.003), which implies that that the hazard increased by 5% _per year_  or 1.05^9^ = 1.55 (155\%) over the 9 years. Under the proportional hazards assumption of this model, this risk doesn't change over time, but we need to formally check this assumption.

 # EXERCISE: While there appears to be a temporal trend in breast-feeding habits, it's possible that it was explained by other factors, such as changes in poverty prevalence, quality of prenatal care, or ethnic mix.

 # full model determined a priori as part of survey design
 model2<-coxph(survival~year + race + poverty + smoke + alcohol + age + yschool + pc3mth, data=d)
 summary(model2)

 # What happens to the impact of _year_ with additional adjustment? How do you interpret the other hazards in the model?
   
 # ### Diagnostics: Testing Proportional Hazards Assumption
 par(mfrow=c(3,3)) # set up plot window 3 rows, 3 cols
 dx=cox.zph(model2); # calculate Schoenberg residuals
 plot(dx) # display test for PH

 # Typically, visual inspection is sufficient to judges whether the residuals confirm the PH assumption (the dashed 95\% CI's make this easier). Purists may want to formally test the the 'horizontalness' of the line with Harrell's rho, which is essentially it's correlation. If p $>$ 0.05, the correlation is zero and PH is sastisfied. That said, I rely primarily on visual inspection of the plots, since even small deviations from the horizontal become statistically significant if n is large enough.


 print(dx) # Harrell's Rho test of residuals

#In the output above, you get a PH test for each $\beta$ and for the overall model (GLOBAL). How do you interpret these results?In addition, we should check for extreme observations and assess linearity in the relationship between the log-hazard and individual predictors. See notes for details


# HOMEWORK ASSIGNMENT

 # In the course folder, you will find a dataset nations.csv (http://www.statsguy.ca/datasets.zip), which contains UN data on infant mortality in 207 countries and per capita gross domestic product (GDP). Economists often choose to work with log-transformed income, and you should notice that the logGDP column contains the natural logarithm of GDP for each country. Don't let this confuse you; you just need to know that the log transformation doesn't affect the direction of any regression relationships, since income and log(income) _always_ increase/ decrease together (see the discussion of this homework assignment in the Primer if you need a high school refresher).

 # Dataset columns:

 # - infant.mortality = infant deaths per 1000 live births
 # - logGDP = natural log per capita gross domestic product (measured in US$)
 # - TFR = total fertility rate (number of children per mother $\equiv$ family size )
 # - contraception = proportion of population with access to family planning programs
 # - region = Africa, Europe, Americas, Asia, or Oceania
 # - nation = country (a label, not a predictor)


 # By treating infant mortality as the dependent variable (y) and logGDP = log (per capita GDP) as the predictor of primary interest, develop a main effects regression model for predicting infant mortality rates though exploratory analysis. Distinguish numeric and categorical predictors. Why can you not use `nation` as a predictor? For easier interpretation, you may want to make "Americas" or "Europe" the reference category for region.

 # Explain in your own words the meaning of the regression coefficients and their p-values

 # What is the relationship between infant mortality and logGDP?

 # What is the relationship between infant mortality and access to family planning?

 # What is the relationship between infant mortality and family size?

 # What is the relationship between infant mortality and geographic region?

 # Are these causal relationships? Why or why not?

 # Explain the meaning of the R^2^ statistic for the fitted model. Did its value surprise you?

 # Bonus 1: Economists often prefer studying the effects of income (e.g. GDP) after first taking the logarithm. Why do you think this is? If you're not sure, you might want to convert `logGDP` back to `GDP` and plot the relationship between infant mortality and per capita GDP. Since this is a natural logarithm, you can recover the original GDP (in US dollars per person) via GDP =  exp(logGDP)

 # Bonus 2: If you're feeling particularly ambitious, perform some basic diagnostic tests to assess model validity e.g. examine the residual plots and exclude problems with collinearity.  What are they telling you?